#INCLUDE	"TI-82.h"		; TI-82 header file
.org		0			; So we can use OS-82 jump macros
.db		"Logic Analyser 1.0",0	; Program title

;               Logic Analyser
;
;    v1.0 for OS-82 - TI-82 Assembly Shell
;
;              By  Vasantha Crabb
;
; (C) 1999
;

;---------------------
; Program VARIABLES
;---------------------

TRIG_MODE	= TEXT_MEM2+$00		; Trigger mode
SAMP_DLY	= TEXT_MEM2+$01		; Delay between samples
MENU_ITEM	= TEXT_MEM2+$02		; The setting being adjusted

ACQ_BUF		= TEXT_MEM2+$03		; Acquisition buffer

;---------------------
; Program CONSTANTS
;---------------------

TRIG_MAN	= 1			; Manual trigger
TRIG_1R		= 2			; Channel 1 rising
TRIG_1F		= 3			; Channel 1 falling
TRIG_2R		= 4			; Channel 2 rising
TRIG_2F		= 5			; Channel 2 falling

; Codes returned from GET_KEY
G_DOWN		= $01			; DOWN key
G_LEFT		= $02			; LEFT key
G_RIGHT		= $03			; RIGHT key
G_UP		= $04			; UP key
G_ENTER		= $09			; ENTER key
G_CLEAR		= $0F			; CLEAR key

;---------------------
; Start of OS-82 PROGRAM
;---------------------

	ld	de,(PROGRAM_ADDR)	; Used for calculating addresses
	ld	hl,GRAPH_MEM		; Clear the Graph Memory
	ld	b,64			; Number of rows
MasterClearLoop:
	ld	c,b			; Nested loop tricks
	ld	b,12			; Number of bytes per row
ClearRowLoop:
	ld	(hl),0			; Clear a byte
	inc	hl			; Increment the data counter
	djnz	ClearRowLoop		; Inner loop
	ld	b,c			; More nested loop tricks
	djnz	MasterClearLoop		; Outer loop

	ld	hl,$8001		; Initialise variables
	ld	(TRIG_MODE),hl

	ld	hl,TitleStr		; Get the address of the string
	add	hl,de
	set	1,(iy+$05)		; Fully frame text
	set	3,(iy+$05)		; White on black
	ld	bc,$0011		; Move to top of screen
	CALL_(ShowStr)			; Display the title string

	res	3,(iy+$05)		; Black on white
	ld	bc,$0712		; Move down a line
	CALL_(ShowStr)			; Display the credit string

	ld	bc,$0E01		; Move the cursor
	CALL_(ShowStr)			; Display the status string

	ld	bc,$1405		; Move the cursor
	CALL_(ShowStr)			; Display the trigger mode string

	ld	bc,$1A05		; Move down a line
	CALL_(ShowStr)			; Display the delay string

	ld	bc,$2201		; Move the cursor
	CALL_(ShowStr)			; Display the Ch.1 string

	ld	bc,$3001		; Move down a line
	CALL_(ShowStr)			; Display the Ch.2 string

	ld	bc,$0E22		; Move up and across
	CALL_(ShowStr)			; Display the Idle string

	CALL_(UpKeyHandler)		; Set menu item to zero

MainLoop:
	ld	hl,StrTrigMan		; Offset to the first mode string
	ld	de,(PROGRAM_ADDR)	; Add the program address
	add	hl,de
	ld	a,(TRIG_MODE)		; Get the trigger mode in a

ChkTrigIdx:
	dec	a			; We're one string forward
	or	a			; If a equals zero..
	jr	z,FoundModeStr		; ..we've found the string
	push	af			; Save that for later

ZSearchLoop:				; Find the next string
	ld	a,(hl)			; Get a byte
	inc	hl			; Advance the data counter
	or	a			; See if the byte's a null
	jr	nz,ZSearchLoop		; Loop if it isn't

	pop	af			; Get the mode index back
	jr	ChkTrigIdx		; See if we're at the right string

FoundModeStr:
	ld	bc,$1422		; Move the cursor
	CALL_(ShowStr)			; Display the string

	ld	a,(SAMP_DLY)		; Display sampling delay
	ld	h,0			; Get the delay into hl
	ld	l,a

	call	UNPACK_HL		; Display the ones digit
	ld	de,$1A2A
	ld	(CURSOR_X),de
	add	a,'0'
	ROM_CALL(M_CHARPUT)

	call	UNPACK_HL		; Display the tens digit
	ld	de,$1A26
	ld	(CURSOR_X),de
	add	a,'0'
	ROM_CALL(M_CHARPUT)

	call	UNPACK_HL		; Display the hundreds digit
	ld	de,$1A22
	ld	(CURSOR_X),de
	add	a,'0'
	ROM_CALL(M_CHARPUT)

	res	4,(iy+$09)		; In case the user hit ON
	ROM_CALL(_GRBUFCPY_V)		; Update the screen

	call	GET_KEY			; Get a key

	cp	G_CLEAR			; User hit CLEAR
	ret	z			; Exit

	cp	G_ENTER			; User says go!
	CALL_Z(Acquire)

	cp	G_UP			; Handle arrow keys
	CALL_Z(UpKeyHandler)

	cp	G_DOWN
	CALL_Z(DownKeyHandler)

	cp	G_RIGHT
	CALL_Z(RightKeyHandler)

	cp	G_LEFT
	CALL_Z(LeftKeyHandler)

	JUMP_(MainLoop)			; Loop

;---------------------
; SUB for running an acquisition
;---------------------

Acquire:
	ld	de,$1401		; Set cursor location
	CALL_(KillMenuMarker)		; Erase upper marker
	ld	de,$1A01		; Set cursor location
	CALL_(KillMenuMarker)		; Erase lower marker

	ld	bc,$0E22		; Tell the user what's going on
	ld	hl,StrStatusArmed	; Get the string offset
	ld	de,(PROGRAM_ADDR)	; Turn it into an address
	add	hl,de
	CALL_(ShowStr)			; Display it

	ROM_CALL(_GRBUFCPY_V)		; Update the screen

	di				; Maximize speed (no interrupts)
	ld	a,(SAMP_DLY)		; Get delay in c'
	ld	c,a
	inc	c			; Fix delay (0 = fastest)
	exx
	ld	b,96			; Number of samples
	ld	hl,ACQ_BUF		; Buffer to store samples in

	ld	a,(TRIG_MODE)		; Check trigger mode
	cp	TRIG_MAN		; Manual trigger starts now
	jr	z,AcqLoop

	ld	c,$01			; Check for Ch.1 Modes
	cp	TRIG_1R
	jr	z,TrigRLpA
	cp	TRIG_1F
	jr	z,TrigFLpA

	ld	c,$02			; Check for Ch.2 Modes
	cp	TRIG_2R
	jr	z,TrigRLpA

TrigFLpA:				; Wait for a one
	in	a,(LINKPORT)
	and	c
	or	a
	jr	z,TrigFLpA
TrigFLpB:				; Wait for a zero
	in	a,(LINKPORT)
	and	c
	or	a
	jr	nz,TrigFLpB
	jr	AcqLoop			; Acquire

TrigRLpA:				; Wait for a zero
	in	a,(LINKPORT)
	and	c
	or	a
	jr	nz,TrigRLpA
TrigRLpB:				; Wait for a one
	in	a,(LINKPORT)
	and	c
	or	a
	jr	z,TrigRLpB

AcqLoop:
	in	a,(LINKPORT)		; Get a byte
	ld	(hl),a			; Store it
	inc	hl			; Increment the data counter
	exx				; Swap the registers over
	ld	b,c			; Get the delay byte into b
AcqDlyLoop:
	djnz	AcqDlyLoop		; Generate the delay 
	exx				; Swap the registers back
	djnz	AcqLoop			; Loop

	ld	bc,$0E22		; Tell the user what's going on
	ld	hl,StrStatusBusy	; Get the string offset
	ld	de,(PROGRAM_ADDR)	; Turn it into an address
	add	hl,de
	CALL_(ShowStr)			; Display it

	ROM_CALL(_GRBUFCPY_V)		; Update the screen

	ld	ix,ACQ_BUF		; Initialise the data counter
	ld	de,$0000		; Initialise the screen counter
	ld	b,12			; Initialise loop counter
SampOutLoop:
	push	bc			; Save loop counter
	push	ix			; Save data counter

	ld	b,8			; Eight samples to the byte
	ld	c,%10000000		; For lighting pixels
	ld	a,0			; Start with a blank byte
Ch1OutLoop:
	ld	h,(ix+$00)		; Get a sample from buffer
	bit	0,h			; See if Ch.1 was high
	jr	z,Ch1LoopNow		; Loop if it wasn't
	or	c			; Light a pixel if it was
Ch1LoopNow:
	srl	c			; Move along one pixel
	inc	ix			; Move along one sample
	djnz	Ch1OutLoop		; Loop

	ld	hl,GRAPH_MEM+$01EC	; Light the 1 line
	add	hl,de
	ld	(hl),a
	cpl				; Complement the pixels
	ld	hl,GRAPH_MEM+$0228	; Light the 0 line
	add	hl,de
	ld	(hl),a

	pop	ix			; Restore data counter
	ld	b,8			; Eight samples to the byte
	ld	c,%10000000		; For lighting pixels
	ld	a,0			; Start with a blank byte
Ch2OutLoop:
	ld	h,(ix+$00)		; Get a sample from buffer
	bit	1,h			; See if Ch.1 was high
	jr	z,Ch2LoopNow		; Loop if it wasn't
	or	c			; Light a pixel if it was
Ch2LoopNow:
	srl	c			; Move along one pixel
	inc	ix			; Move along one sample
	djnz	Ch2OutLoop		; Loop

	ld	hl,GRAPH_MEM+$0294	; Light the 1 line
	add	hl,de
	ld	(hl),a
	cpl				; Complement the pixels
	ld	hl,GRAPH_MEM+$02D0	; Light the 0 line
	add	hl,de
	ld	(hl),a

	pop	bc			; Restore loop counter
	inc	de			; Increment screen counter
	djnz	SampOutLoop		; Loop

	ld	bc,$0E22		; Tell the user what's going on
	ld	hl,StrStatusIdle	; Get the string offset
	ld	de,(PROGRAM_ADDR)	; Turn it into an address
	add	hl,de
	CALL_(ShowStr)			; Display it

	ei				; We'll need the interrupts later
	CALL_(UpKeyHandler)		; Fix the menu
	ret				; Return

;---------------------
; SUB for showing the ASCIIZ string at (hl) at location (c,b)
;---------------------

ShowStr:
	ld	(CURSOR_X),bc		; Set the cursor location
	ROM_CALL(D_ZM_STR)		; Display the string
	ret				; Return

;---------------------
; SUB for handling the up arrow key
;---------------------

UpKeyHandler:
	ld	de,$1A01			; Set cursor location
	CALL_(KillMenuMarker)			; Erase old marker

	ld	de,$1401			; Move cursor
	ld	(CURSOR_X),de
	ld	a,$05				; Display new marker
	ROM_CALL(M_CHARPUT)

	ld	a,0				; Set the menu item
	ld	(MENU_ITEM),a
	ret					; Return

;---------------------
; SUB for handling the down arrow key
;---------------------

DownKeyHandler:
	ld	a,1				; Set the menu item
	ld	(MENU_ITEM),a

	ld	de,$1401			; Set cursor location
	CALL_(KillMenuMarker)			; Erase old marker

	ld	de,$1A01			; Move cursor
	ld	(CURSOR_X),de
	ld	a,$05				; Display new marker
	ROM_CALL(M_CHARPUT)

	sub	a
	ret					; Return

;---------------------
; SUB for handling the right arrow key
;---------------------

RightKeyHandler:
	ld	a,(MENU_ITEM)			; See which item we're on
	or	a
	jr	z,IncTrigMode			; Item zero is the trigger
	jr	nz,IncSampDly			; Item one is the delay

;---------------------
; SUB for handling the left arrow key
;---------------------

LeftKeyHandler:
	ld	a,(MENU_ITEM)			; See which item we're on
	or	a
	jr	z,DecTrigMode			; Item zero is the trigger
	jr	nz,DecSampDly			; Item one is the delay

;---------------------
; SUB for incrementing the trigger mode
;---------------------

IncTrigMode:
	ld	a,(TRIG_MODE)			; Get the current mode in a
	cp	TRIG_2F				; Return if it's Ch.2 Neg.
	ret	z
	inc	a				; Increment it
	ld	(TRIG_MODE),a			; Store new mode
	sub	a
	ret					; Return

;---------------------
; SUB for decrementing the trigger mode
;---------------------

DecTrigMode:
	ld	a,(TRIG_MODE)			; Get the current mode in a
	dec	a				; Decrement it
	or	a				; Return if it's zero
	ret	z
	ld	(TRIG_MODE),a			; Store new mode
	sub	a
	ret					; Return

;---------------------
; SUB for incrementing the sampling delay
;---------------------

IncSampDly:
	ld	a,(SAMP_DLY)
	inc	a
	ld	(SAMP_DLY),a
	sub	a
	ret					; Return

;---------------------
; SUB for decrementing the sampling delay
;---------------------

DecSampDly:
	ld	a,(SAMP_DLY)
	dec	a
	ld	(SAMP_DLY),a
	sub	a
	ret					; Return

;---------------------
; SUB for hiding the menu marker
;---------------------

KillMenuMarker:
	ld	(CURSOR_X),de			; Set the cursor location
	ld	b,3
KillMarkerLoop:
	ld	a,' '				; Erase a line
	ROM_CALL(M_CHARPUT)
	djnz	KillMarkerLoop			; Loop
	ret					; Return

;---------------------
; Program DATA
;---------------------

TitleStr:
	.db	" Logic Analyser 1.0",0
	.db	"by Vasantha Crabb",0

StatusStr:				; Strings on the main screen
	.db	"Status:",0
TrigModeStr:
	.db	"Trigger:",0
DelayStr:
	.db	"Delay:",0
Ch1Str:
	.db	"Channel 1 (Tip)",0
Ch2Str:
	.db	"Channel 2 (Ring)",0

StrStatusIdle:				; Status display strings
	.db	"Idle"
	.db	"                    "
	.db	"                   ",0
StrStatusArmed:
	.db	"Trigger Armed",0
StrStatusBusy:
	.db	"Processing Data",0

StrTrigMan:				; Trigger mode strings
	.db	"Manual"
	.db	"                          ",0
StrTrig1R:
	.db	"Channel 1 Rising     ",0
StrTrig1F:
	.db	"Channel 1 Falling",0
StrTrig2R:
	.db	"Channel 2 Rising     ",0
StrTrig2F:
	.db	"Channel 2 Falling",0

.end
